### [Unreleased]
{% if new_version %}
### [{{ new_version }}] - {{ today }}
{% endif %}
{% for name, notes in sections.items() -%}
{% if notes -%}
#### {{ name.title() }}
{% for note in notes -%}
- {{ note }}
{% endfor %}
{% endif %}

{%- endfor -%}
### [{{ old_version }}]
