<!--
  !! THIS FILE IS MAINTAINED USING CHANGELOGGER.
  !! MODIFICATION OF THIS FILE BY HAND MAY BREAK USAGE WITH CHANGELOGGER.

  Learn more: https://github.com/award28/changelogger
-->
# Changelog
*All notable changes to this project will be documented in this file.

The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),
and this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).
The [Changelogger tool](https://pypi.org/project/changelogged) is used for automated management of this file.

<!-- BEGIN RELEASE NOTES -->
### [Unreleased]

### [0.1.0] - {{ today }}
<!-- END RELEASE NOTES -->
<!-- BEGIN LINKS -->
[Unreleased]: https://github.com/{{ context.git.repo }}/compare/0.1.0...HEAD
[0.1.0]: https://github.com/{{ context.git.repo }}/compare/{{ context.git.first_commit }}...0.1.0
<!-- END LINKS -->
