[Unreleased]: https://github.com/{{ context.git.repo }}/compare/{{ new_version }}...HEAD
[{{ new_version }}]: https://github.com/{{ context.git.repo }}/compare/{{ old_version }}...{{ new_version }}
{{ '' }}
